#ifndef _FILETREE_H_
#define _FILETREE_H_

#include "pixmaps.h"

#include <qlistview.h>
#include <qpopupmenu.h>

class FileTree : public QListView
{
  Q_OBJECT
protected:
  void downloadFile(QString target_dir);
  void downloadDir(QListViewItem *, QString target_dir, bool recursive);

public:
  /// Constructor. Calls parent constructor, creates some columns
  /// and connects some signals to get the menu working.
  FileTree( QWidget* parent = 0, const char* name = 0, bool withGetFileList = true);

public slots:
  void getUserFileList();

  void slot_downloadFile();
  void slot_downloadDir();
  void slot_downloadDirRecursive();
  void slot_downloadFileTo();
  void slot_downloadDirTo();
  void slot_downloadDirRecursiveTo();
  
  void dispMenu( QListViewItem* item, const QPoint& p, int column );
  void addItemToTree( const QString& user, 
			 const QString& fileName, 
			 const QString& size,
			 const QString& slot);

signals:
  void getUserFileList( const QString& nick );
  void downloadFile( const QString& nick, 
		     const QString& fileName, 
		     const QString& localName,
		     unsigned long int size );
    

private:

  QPopupMenu* menu;
  QPopupMenu* dirMenu;

};

#endif


/*
 *
 * $Log: filetree.h,v $
 * Revision 1.4  2004/02/18 07:58:42  estrato
 * download to dir, resume download, bugfixes
 *
 * Revision 1.3  2003/12/08 00:13:40  estrato
 * Directory downloads
 *
 * Revision 1.2  2003/08/18 12:19:43  estrato
 * Old transfer dialog now completely replaced, but some functionality is gone. Will add complete functionality in next commit.
 *
 * Revision 1.1.1.1  2003/06/24 10:16:18  olof
 * remake of cvs tree
 *
 * Revision 1.6  2003/06/16 12:18:12  olof
 * loads
 *
 * Revision 1.5  2003/06/11 22:14:21  olof
 * transfers ok again
 *
 * Revision 1.4  2003/05/07 00:11:54  olof
 * massa mandelmassa
 *
 * Revision 1.3  2003/04/16 20:54:47  olof
 * user file list fixed
 *
 * Revision 1.2  2003/03/18 14:27:29  mickeg
 * Filetree working
 * Hopefully the userfilelisting also works.
 *
 * Revision 1.1  2003/03/16 21:00:48  mickeg
 * Added functionality for the tree view of the search results
 * Still some bugs
 * Doesn't iterate through the children accurately
 *
 *
 */
