#include "status_monitor.h"
#include "pixmaps.h"
#include "dc_settings.h"
#include "debugdlg.h"

#include <qbitmap.h>
#include <qpainter.h>

status_monitor *statMon = NULL;

// ms before showing status info tip when mouse hovers over the icon
#define INFOTIP_TIMEOUT 1000

// max time the info tip remains on screen
#define INFOTIP_TTL 10000


status_monitor::status_monitor(QWidget *parent, const char *name) :
  QFrame(parent, name, Qt::WStyle_Customize | Qt::WStyle_NoBorder),
  n_hubs(0),
  n_uls(0),
  n_qgdls(0),
  n_agdls(0),
  n_fgdls(0),
  b_gui_shown(true),
  b_showing_popup(false),
  b_popup_visible(false),
  b_has_news(false)
{
  Pixmap *pix = new Pixmap;
  pIcon = pix->getMinimode();
  pIconFull = pix->getMinimodeFull();
  pOn = pix->getMinimode_on();
  pUL = pix->getMinimode_ul();
  pDL = pix->getMinimode_dl();
  QBitmap tmp;
  tmp = pix->getMinimode_alpha();
  setMask(tmp);
  delete pix;

  infotip_timer = new QTimer(this);
  QObject::connect(infotip_timer, SIGNAL(timeout()),
		   this, SLOT(slot_show_popup()));

  info_ttl_timer = new QTimer(this);
  QObject::connect(info_ttl_timer, SIGNAL(timeout()),
		   this, SLOT(slot_hide_popup()));

  setMouseTracking(true);

  popup = new QFrame(0, "status info popup", Qt::WStyle_Customize|Qt::WStyle_NoBorder);
  popup->setFrameStyle( QFrame::NoFrame );
//  popup->setFixedSize(200,110);

  box = new QVGroupBox(popup, "popbox");
//  box->setFixedSize(200,110);

  ln_hubs = new QLabel(box, "numhubs"); ln_hubs->setText("");
  ln_qgdls = new QLabel(box, "numqgdls"); ln_qgdls->setText("");
  ln_agdls = new QLabel(box, "numagdls"); ln_agdls->setText("");
//  ln_fgdls = new QLabel(box, "numfgdls"); ln_fgdls->setText("");
  ln_uls = new QLabel(box, "numuls"); ln_uls->setText("");

  popup->hide();

  setFixedSize(75, 75);

  move(QPoint(20,20));
  statMon = this;
}


status_monitor::~status_monitor()
{
  delete ln_hubs;
  delete popup;
  statMon = NULL;
}


void status_monitor::paintEvent( QPaintEvent * e )
{
  QPainter p( this );
  p.setPen(NoPen);
  p.setBrush(NoBrush);

  if (b_has_news)
    p.drawPixmap( 0, 0, pIconFull );
  else
    p.drawPixmap( 0, 0, pIcon );

  if (n_hubs>0)
    p.drawPixmap( 20,33, pOn );

  if (n_agdls>0)
    p.drawPixmap( 46,37, pDL );

  if (n_uls>0)
    p.drawPixmap( 53,36, pUL );
}


void status_monitor::mouseMoveEvent ( QMouseEvent * e )
{
  if (hasMouse()) {
    if (!b_showing_popup) {
      if (infotip_timer->isActive())
	infotip_timer->stop();
      infotip_timer->start(INFOTIP_TIMEOUT, true);
      b_showing_popup=true;
      debugWin->print("Will show status info after a delay...");
    }
  } else {
    if (b_popup_visible)
      slot_hide_popup();
  }
}

void status_monitor::mouseDoubleClickEvent ( QMouseEvent * e )
{
  if (e->button()==LeftButton) {
    debugWin->print("statMon: mouseDoubleClickEvent() -- LeftB");
    if (b_popup_visible)
      slot_hide_popup();
    slot_show_ui();
  }
}

void status_monitor::mousePressEvent( QMouseEvent * e )
{
  if (e->button()==LeftButton) {
    debugWin->print("statMon: mousePressEvent() -- LeftB");
    if (b_popup_visible)
      slot_hide_popup();
  } else if (e->button()==RightButton) {
    debugWin->print("statMon: mousePressEvent() -- RightB");
    if (b_popup_visible)
      slot_hide_popup();
    else {
      if (!b_showing_popup) {
	if (infotip_timer->isActive())
	  infotip_timer->stop();
	infotip_timer->start(INFOTIP_TIMEOUT, true);
	b_showing_popup=true;
	debugWin->print("Will show status info after a delay...");
      }
    }
  }
}


void status_monitor::leaveEvent(QEvent *e)
{
  if (b_popup_visible)
    slot_hide_popup();
}


void status_monitor::slot_show_popup()
{
  if (!hasMouse() || !b_showing_popup) {
    b_showing_popup = false;
    return;
  }

  b_popup_visible = true;

  ln_hubs->setText(QString("Open hubs:  %1").arg(n_hubs)); ln_hubs->adjustSize();
  ln_qgdls->setText(QString("Files in queue:  %1").arg(n_qgdls)); ln_qgdls->adjustSize();
  ln_agdls->setText(QString("Downloads:  %1").arg(n_agdls)); ln_agdls->adjustSize();
//  ln_fgdls->setText(QString("Finished downloads:  %1").arg(n_fgdls)); ln_fgdls->adjustSize();
  ln_uls->setText(QString("Uploads:  %1").arg(n_uls)); ln_uls->adjustSize();

  box->adjustSize();

  popup->move(QPoint(x(), y()+height()));
  popup->adjustSize();
  popup->update();
  popup->show();

  if (info_ttl_timer->isActive())
    info_ttl_timer->stop();
  info_ttl_timer->start(INFOTIP_TTL, true);

  debugWin->print("Showing status info popup. Will hide after delay.");
}


void status_monitor::slot_hide_popup()
{
  b_showing_popup = false;
  b_popup_visible = false;

  if (infotip_timer->isActive())
    infotip_timer->stop();

  if (info_ttl_timer->isActive())
    info_ttl_timer->stop();

  popup->hide();

  debugWin->print("Hiding status info popup.");
}


void status_monitor::slot_show_ui()
{
  int px=x(), py=y();
  settings.set_setting("common", "iconmode_pos_x", QString("%1").arg(px));
  settings.set_setting("common", "iconmode_pos_y", QString("%1").arg(py));
  settings.save_to_file();

  b_gui_shown = true;
  b_has_news = false;
  slot_hide_popup();
  this->hide();
  emit sig_show_ui();
}


void status_monitor::slot_hide_ui()
{
  bool cvok=false;
  int px, py;
  px = settings.get_setting("iconmode_pos_x").toInt(&cvok);
  if (!cvok) px = 20;
  py = settings.get_setting("iconmode_pos_y").toInt(&cvok);
  if (!cvok) py = 20;
  move(QPoint(px, py));

  b_gui_shown = false;
  b_has_news = false;
  slot_hide_popup();
  this->show();
  emit sig_hide_ui();
}


void status_monitor::hub_opened()
{
  debugWin->print("statMon: hub_opened()");
  n_hubs++;

  if (!b_gui_shown && n_hubs==1)
    update();
}

void status_monitor::hub_closed()
{
  debugWin->print("statMon: hub_closed()");
  if (n_hubs>0)
    n_hubs--;

  if (!b_gui_shown && n_hubs==0)
    update();
}


void status_monitor::ul_added()
{
  debugWin->print("statMon: ul_added()");
  n_uls++;

  if (!b_gui_shown && n_uls==1)
    update();
}

void status_monitor::ul_removed()
{
  debugWin->print("statMon: ul_removed()");
  if (n_uls>0)
    n_uls--;
  if (!b_gui_shown && n_uls==0)
    update();
}


void status_monitor::gdl_added(const unsigned int id,
			       const QString &fname,
			       const unsigned long int fsize)
{
  debugWin->print("statMon: gdl_added()");
  gdls[id] = gdl_t(fname, fsize);
  n_qgdls++;
}

void status_monitor::gdl_removed(const unsigned int id)
{
  debugWin->print("statMon: gdl_removed()");
  if (gdls.find(id) != gdls.end()) {
    gdls.erase(gdls.find(id));
    if (n_qgdls>0) {
      n_qgdls--;
      n_fgdls++;
    }
  }

  if (!b_gui_shown && !b_has_news) {
    b_has_news = true;
    update();
  }
}


void status_monitor::gdl_queued()
{
  debugWin->print("statMon: gdl_queued()");
  if (n_agdls>0)
    n_agdls--;

  if (!b_gui_shown && n_agdls==0)
    update();
}

void status_monitor::gdl_active()
{
  debugWin->print("statMon: gdl_active()");
  n_agdls++;
  if (!b_gui_shown && n_agdls==1)
    update();
}


int status_monitor::num_open_hubs() const
{
  return n_hubs;
}

int status_monitor::num_uls() const
{
  return n_uls;
}

int status_monitor::num_queued_gdls() const
{
  return n_qgdls;
}

int status_monitor::num_active_gdls() const
{
  return n_agdls;
}

unsigned int status_monitor::get_gdl_id_first(const unsigned long int fsize)
{
  return 0;
}

unsigned int status_monitor::get_gdl_id_next(const unsigned long int fsize)
{
  return 0;
}

QString status_monitor::get_gdl_name_by_id(const unsigned int id) const
{
  return QString("");
}


